Imports System.IO
''' <summary>
''' Mit dieser Klasse knnen sie Informationen
''' zu einem Ordner ermittelen, die sie ber
''' ein DirectoryInfo Objekt nicht abrufen knnen.
''' </summary>
''' <remarks></remarks>
Public Class DirectoryInfoExtended
    'Interne Variablen zur Speicherung
    'Rufen sie den Wert ber die
    'entsprechende Eigenschaft ab.
    Private m_Directory As DirectoryInfo
    Private m_Initalized As Boolean
    Private m_ThrowErrors As Boolean
    Private m_DirCount As Long
    Private m_FileCount As Long
    Private m_FullSize As Long

    ''' <summary>
    ''' Gibt den zu verarbeitenden Ordner als
    ''' Directory-Info Objekt zurck.
    ''' </summary>
    Public ReadOnly Property Directory() As DirectoryInfo
        Get
            Return m_Directory
        End Get
    End Property

    ''' <summary>
    ''' Diese Eigenschaft gibt an, ob die zustzlichen
    ''' Informationen bereit ermittelt wurden.
    ''' </summary>
    Public ReadOnly Property Initalized() As Boolean
        Get
            Return m_Initalized
        End Get
    End Property

    ''' <summary>
    ''' Diese Eigenschaft gibt an, ob auftretende
    ''' Exception abgefeuert werden sollen oder nicht.
    ''' </summary>
    ''' <remarks>UnauthorizedAccessException werden
    ''' immer abgefangen.</remarks>
    Public ReadOnly Property ThrowErrors() As Boolean
        Get
            Return m_ThrowErrors
        End Get
    End Property

    ''' <summary>
    ''' Diese Eigenschaft gibt an, wie viele Ordner, der
    ''' in Directory angebene Ordner enthlt.
    ''' </summary>
    ''' <remarks>Liegen die Informationen noch nicht vor 
    ''' gibt die Eigenschaft '-1' zurck.</remarks>
    Public ReadOnly Property DirCount() As Long
        Get
            Return m_DirCount
        End Get
    End Property

    ''' <summary>
    ''' Diese Eigenschaft gibt an, wie viele Dateien, der
    ''' in Directory angebene Ordner enthlt.
    ''' </summary>
    ''' <remarks>Liegen die Informationen noch nicht vor 
    ''' gibt die Eigenschaft '-1' zurck.</remarks>
    ReadOnly Property FileCount() As Long
        Get
            Return m_FileCount
        End Get
    End Property

    ''' <summary>
    ''' Diese Eigenschaft gibt an, wie gro alle Dateien
    ''' sind, die der in Directory angebene Ordner enthlt.
    ''' </summary>
    ''' <remarks>Liegen die Informationen noch nicht vor 
    ''' gibt die Eigenschaft '-1' zurck. Die Funktion
    ''' addiert die Dateigren, der tatschliche Speicherbedarf
    ''' kann allerdings grer oder kleiner sein.</remarks>
    Public ReadOnly Property FullSize() As Long
        Get
            Return m_FullSize 
        End Get
    End Property

    ''' <summary>
    ''' Erstellt eine neue Instanz, die Informationen
    ''' zu einem Ordner ermittelen, welche sie ber
    ''' ein DirectoryInfo Objekt nicht abrufen knnen.
    ''' </summary>
    ''' <param name="Dir">Der zu verarbeitende Ordner</param>
    ''' <param name="Synchron">Gibt an, ob die Informationen
    ''' im gleichen Thread (synchron) oder in einem eigenen
    ''' Thread (asynchron) ermittelt werden sollen.</param>
    ''' <param name="ThrowErr">Gibt an, ob auftretende
    ''' Exception abgefeuert werden sollen oder nicht.</param>
    ''' <remarks>Sollten sie einen asynchronen Aufruf whlen,
    ''' knnen sie ber die Eigenschaft Initalized prfen,
    ''' ob die Informationen bereits vorliegen.</remarks>
    Public Sub New(ByVal Dir As String, _
     ByVal Synchron As Boolean, ByVal ThrowErr As Boolean)
        m_ThrowErrors = ThrowErr
        m_Directory = New DirectoryInfo(Dir)
        'Ermittelt die erweiterten Informationen
        GetInfo(Synchron)
    End Sub

    ''' <summary>
    ''' Erstellt eine neue Instanz, die Informationen
    ''' zu einem Ordner ermittelen, welche sie ber
    ''' ein DirectoryInfo Objekt nicht abrufen knnen.
    ''' </summary>
    ''' <param name="Dir">Der zu verarbeitende Ordner</param>
    ''' <param name="Synchron">Gibt an, ob die Informationen
    ''' im gleichen Thread (synchron) oder in einem eigenen
    ''' Thread (asynchron) ermittelt werden sollen.</param>
    ''' <param name="ThrowErr">Gibt an, ob auftretende
    ''' Exception abgefeuert werden sollen oder nicht.</param>
    ''' <remarks>Sollten sie einen asynchronen Aufruf whlen,
    ''' knnen sie ber die Eigenschaft Initalized prfen,
    ''' ob die Informationen bereits vorliegen.</remarks>
    Public Sub New(ByVal Dir As DirectoryInfo, _
     ByVal Synchron As Boolean, ByVal ThrowErr As Boolean)
        m_ThrowErrors = ThrowErr
        m_Directory = Dir
        'Ermittelt die erweiterten Informationen
        GetInfo(Synchron)
    End Sub

    'Ermittelt Informationen zu einem Ordner,
    'welche sie ber ein DirectoryInfo Objekt nicht
    'abrufen knnen.
    Private Sub GetInfo(ByVal Synchron As Boolean)
        'Initalisierungsarbeiten
        m_DirCount = -1
        m_FileCount = -1
        m_FullSize = -1
        m_Initalized = False
        'Eine Delegate der die Funktion aufruft
        Dim caller As New AsyncFunctionCaller(AddressOf Calc)

        If Synchron Then
            'Normaler Aufruf mittles Delegate
            'Rckgabe der Funktion wird gespeichert
            Dim result As Info = caller.Invoke(m_Directory, True)
            'Informationen eintragen
            m_DirCount = result.DirCount
            m_FileCount = result.FileCount
            m_FullSize = result.FullSize
        Else
            'Asynchroner Aufruf mit Speicherung der Rckgabe
            'bergabe der Parameter fr die eigentliche Funktion
            'bergabe einer Rckrufmethode
            caller.BeginInvoke(m_Directory, True, AddressOf CallbackMethod, caller)
        End If
    End Sub

    ''' <summary>
    ''' Dieser Delegate dient zum Aufruf (synchron /
    ''' asynchron) der Informationsermittlung.
    ''' </summary>
    ''' <param name="Start">Der zu verarbeitende Ordner</param>
    ''' <param name="SubDirs">Gibt an, ob Unterordner
    ''' bercksichtigt werden sollen.</param>
    Private Delegate Function AsyncFunctionCaller(ByVal Start As IO.DirectoryInfo, _
          ByVal SubDirs As Boolean) As Info

    ''' <summary>
    ''' Dieses Ereignis tritt auf, sobald die erweiterten
    ''' Informationen ermittelt wurden. Initalized gibt daher
    ''' True zurck.
    ''' </summary>
    ''' <param name="ExtDirInfo">Der Ordner zu dem die
    ''' erweiterten Informationen ermittelt wurden.</param>
    ''' <remarks><paramref name="ExtDirInfo"/> verweist auf
    ''' die Instanz fr die die erweiterten Informationen
    ''' ermittelt wurden. Das ist vor allem bei mehreren
    ''' Instanzen hilfreich. Das Ereignis wird nur bei
    ''' Beendigung nach einem asynchronen Aufruf ausgelst.
    ''' </remarks>
    Public Event InitalizingFinished(ByVal ExtDirInfo As DirectoryInfoExtended)

    ''' <summary>
    ''' Diese Funktion wird automatisch aufgerufen
    ''' nachdem der Aufruf zur Informationsermittlung
    ''' abgeschlossen wurde und dient zur Auswertung
    ''' der Informationen und dem Auslsen der Ereignisse.
    ''' </summary>
    ''' <param name="ar">Standardmiger Parameter einer
    ''' so genannten CallBack-Funktion</param>
    Private Sub CallbackMethod(ByVal ar As IAsyncResult)
        'Notwendig damit der asynchrone Aufruf
        'beendet wird und das Ergebnis abgerufen
        'werden kann
        Dim caller As AsyncFunctionCaller = _
            CType(ar.AsyncState, AsyncFunctionCaller)
        Dim result As Info = caller.EndInvoke(ar)
        'Informationen eintragen
        m_DirCount = result.DirCount
        m_FileCount = result.FileCount
        m_FullSize = result.FullSize
        'Arbeit beendet
        'Das dem Benutzer mitteilen
        m_Initalized = True
        RaiseEvent InitalizingFinished(Me)
    End Sub

    'Interne Funktion zur Informationsermittlung
    'zu einem Ordner (erledigt die eigentliche Arbeit).
    'Funktion ist rekursiv um die erweiterten
    'Informationen gegenfalls ber Unterordner zu ermitteln.
    Private Function Calc(ByVal Start As IO.DirectoryInfo, _
      Optional ByVal SubDirs As Boolean = True) As Info
        Try
            'Zum For ... Each aller Ordner
            'Funktion 'GetDirectories' muss nur einmal
            'aufgerufen werden
            Dim dirs() As IO.DirectoryInfo = Start.GetDirectories
            Dim dir As IO.DirectoryInfo
            'Zum For ... Each aller Dateien
            'Funktion 'GetFiles' muss nur einmal
            'aufgerufen werden
            Dim files() As IO.FileInfo = Start.GetFiles
            Dim file As IO.FileInfo
            'Anzahl der Unterordner ermitteln und speichern
            Dim c_Dir As Long = dirs.GetLength(0)
            'Anzahl der Dateien ermitteln und speichern
            Dim c_File As Long = files.GetLength(0)
            'Die Gre aller Dateien speichern
            Dim c_Size As Long
            For Each file In files
                c_Size += file.Length
            Next
            'Temporre Speicherung um Unterordner
            'addieren zu knnen
            Dim temp As Info = New Info(c_Dir, c_File, c_Size)
            'Eventuell Unterordner bercksichtigen
            If SubDirs Then
                For Each dir In dirs
                    'Jeden Unterordner durchlaufen und
                    'ebenfalls enumerieren (Rekursion !)
                    temp += Calc(dir, SubDirs)
                Next
            End If
            'Rckgabe
            Return temp
        Catch ex As System.UnauthorizedAccessException
            'Dieser Error soll abgefangen werden
        Catch ex As System.IO.DirectoryNotFoundException
            'Der Error wird abgefangen aber bei Bedarf
            'gefeuert.
            If m_ThrowErrors Then Throw ex
        Catch ex As System.IO.DriveNotFoundException
            'Der Error wird abgefangen aber bei Bedarf
            'gefeuert.
            If m_ThrowErrors Then Throw ex
        Catch ex As System.IO.FileNotFoundException
            'Der Error wird abgefangen aber bei Bedarf
            'gefeuert.
            If m_ThrowErrors Then Throw ex
        End Try
    End Function

    'Mit dieser Struktur knnen alle erweiterten
    'Informationen an die aufrufende Funktion
    'zurckgegeben werden. Da die Struktur nur
    'internen Zwecken dient, wurde verzichtet
    'Variablen als Privat zu deklarieren und 
    'durch eine Eigenschaft zuggnlich zu machen.
    Private Structure Info
        'Die Anzahl der enthaltenen Ordner
        Public DirCount As Long
        'Die Anzahl der enthaltenen Dateien
        Public FileCount As Long
        'Die Gre der enthaltenen Dateien
        Public FullSize As Long

        'Hiermit knnen sie eine neue
        'Instanz der Struktur erstellen und sofort
        'alle Informationen initalisieren.
        Sub New(ByVal p_DirCount As Long, _
         ByVal p_FileCount As Long, _
         ByVal p_FullSize As Long)
            Me.DirCount = p_DirCount
            Me.FileCount = p_FileCount
            Me.FullSize = p_FullSize
        End Sub

        ''' <summary>
        ''' Diese Funktion addiert zwei Strukturen
        ''' und gibt die Summe aus <paramref name="d1"/>
        ''' und <paramref name=" d2"/> zurck.
        ''' </summary>
        ''' <param name="d1">Die erste Struktur
        ''' des Additionsvorgangs</param>
        ''' <param name="d2">Die zweite Struktur
        ''' des Additionsvorgangs</param>
        ''' <remarks>Diese Funktion dient dem so
        ''' genannten Operator Overloading. Somit
        ''' knnen mehrere Ordner ohne grere
        ''' Umwege direkt zusammengefasst werden.</remarks>
        Shared Operator +(ByVal d1 As Info, ByVal d2 As Info) As Info
            Return New Info(d1.DirCount + d2.DirCount, _
                            d1.FileCount + d2.FileCount, _
                            d1.FullSize + d2.FullSize)

        End Operator
    End Structure
End Class
